import datetime
from _typeshed import Unused
from collections import OrderedDict
from collections.abc import Generator
from re import Match, Pattern
from typing import Any, Final, Generic, Literal, Protocol, TypeVar, overload, type_check_only
from typing_extensions import Never, Self, TypeAlias

_R_co = TypeVar("_R_co", float, datetime.datetime, default=float, covariant=True)
_R2_co = TypeVar("_R2_co", float, datetime.datetime, covariant=True)
_Expressions: TypeAlias = list[str]  # fixed-length list of 5 or 6 strings

@type_check_only
class _AllIter(Protocol[_R_co]):
    @overload
    def __call__(
        self, ret_type: type[_R2_co], start_time: float | datetime.datetime | None = None, update_current: bool | None = None
    ) -> Generator[_R2_co]: ...
    @overload
    def __call__(
        self, ret_type: None = None, start_time: float | datetime.datetime | None = None, update_current: bool | None = None
    ) -> Generator[_R_co]: ...

def is_32bit() -> bool: ...

OVERFLOW32B_MODE: Final[bool]

UTC_DT: Final[datetime.timezone]
EPOCH: Final[datetime.datetime]
M_ALPHAS: Final[dict[str, int]]
DOW_ALPHAS: Final[dict[str, int]]

MINUTE_FIELD: Final = 0
HOUR_FIELD: Final = 1
DAY_FIELD: Final = 2
MONTH_FIELD: Final = 3
DOW_FIELD: Final = 4
SECOND_FIELD: Final = 5
YEAR_FIELD: Final = 6

UNIX_FIELDS: Final[tuple[int, int, int, int, int]]
SECOND_FIELDS: Final[tuple[int, int, int, int, int, int]]
YEAR_FIELDS: Final[tuple[int, int, int, int, int, int, int]]

step_search_re: Final[Pattern[str]]
only_int_re: Final[Pattern[str]]

WEEKDAYS: Final[str]
MONTHS: Final[str]
star_or_int_re: Final[Pattern[str]]
special_dow_re: Final[Pattern[str]]
re_star: Final[Pattern[str]]
hash_expression_re: Final[Pattern[str]]

CRON_FIELDS: Final[dict[str | int, tuple[int, ...]]]
UNIX_CRON_LEN: Final = 5
SECOND_CRON_LEN: Final = 6
YEAR_CRON_LEN: Final = 7
VALID_LEN_EXPRESSION: Final[set[int]]
TIMESTAMP_TO_DT_CACHE: Final[dict[tuple[float, str], datetime.datetime]]
EXPRESSIONS: dict[tuple[str, bytes], _Expressions]
MARKER: object

def timedelta_to_seconds(td: datetime.timedelta) -> float: ...
def datetime_to_timestamp(d: datetime.datetime) -> float: ...

class CroniterError(ValueError): ...
class CroniterBadTypeRangeError(TypeError): ...
class CroniterBadCronError(CroniterError): ...
class CroniterUnsupportedSyntaxError(CroniterBadCronError): ...
class CroniterBadDateError(CroniterError): ...
class CroniterNotAlphaError(CroniterError): ...

class croniter(Generic[_R_co]):
    MONTHS_IN_YEAR: Final = 12
    RANGES: Final[
        tuple[
            tuple[int, int], tuple[int, int], tuple[int, int], tuple[int, int], tuple[int, int], tuple[int, int], tuple[int, int]
        ]
    ]
    DAYS: Final[
        tuple[
            Literal[31],
            Literal[28],
            Literal[31],
            Literal[30],
            Literal[31],
            Literal[30],
            Literal[31],
            Literal[31],
            Literal[30],
            Literal[31],
            Literal[30],
            Literal[31],
        ]
    ]
    ALPHACONV: Final[
        tuple[
            dict[Never, Never],
            dict[Never, Never],
            dict[str, str],
            dict[str, int],
            dict[str, int],
            dict[Never, Never],
            dict[Never, Never],
        ]
    ]
    LOWMAP: Final[
        tuple[
            dict[Never, Never],
            dict[Never, Never],
            dict[int, int],
            dict[int, int],
            dict[int, int],
            dict[Never, Never],
            dict[Never, Never],
        ]
    ]
    LEN_MEANS_ALL: Final[tuple[int, int, int, int, int, int, int]]

    second_at_beginning: bool
    tzinfo: datetime.tzinfo | None

    # Initialized to None, but immediately set to a float.
    start_time: float
    dst_start_time: float
    cur: float

    expanded: list[list[str]]
    nth_weekday_of_month: dict[str, set[int]]
    fields: tuple[int, ...]
    expressions: _Expressions

    @overload
    def __new__(
        cls,
        expr_format: str,
        start_time: float | datetime.datetime | None = None,
        ret_type: type[float] = ...,
        day_or: bool = True,
        max_years_between_matches: int | None = None,
        is_prev: bool = False,
        hash_id: str | bytes | None = None,
        implement_cron_bug: bool = False,
        second_at_beginning: bool | None = None,
        expand_from_start_time: bool = False,
    ) -> croniter[float]: ...
    @overload
    def __new__(
        cls,
        expr_format: str,
        start_time: float | datetime.datetime | None,
        ret_type: type[datetime.datetime],
        day_or: bool = True,
        max_years_between_matches: int | None = None,
        is_prev: bool = False,
        hash_id: str | bytes | None = None,
        implement_cron_bug: bool = False,
        second_at_beginning: bool | None = None,
        expand_from_start_time: bool = False,
    ) -> croniter[datetime.datetime]: ...
    @overload
    def __new__(
        cls,
        expr_format: str,
        *,
        ret_type: type[datetime.datetime],
        day_or: bool = True,
        max_years_between_matches: int | None = None,
        is_prev: bool = False,
        hash_id: str | bytes | None = None,
        implement_cron_bug: bool = False,
        second_at_beginning: bool | None = None,
        expand_from_start_time: bool = False,
    ) -> croniter[datetime.datetime]: ...
    def __init__(
        self,
        expr_format: str,
        start_time: float | datetime.datetime | None = None,
        ret_type: type[_R_co] = ...,
        day_or: bool = True,
        max_years_between_matches: int | None = None,
        is_prev: bool = False,
        hash_id: str | bytes | None = None,
        implement_cron_bug: bool = False,
        second_at_beginning: bool | None = None,
        expand_from_start_time: bool = False,
    ) -> None: ...
    @overload
    def get_next(
        self, ret_type: type[_R2_co], start_time: float | datetime.datetime | None = None, update_current: bool = True
    ) -> _R2_co: ...
    @overload
    def get_next(
        self, ret_type: None = None, start_time: float | datetime.datetime | None = None, update_current: bool = True
    ) -> _R_co: ...
    @overload
    def get_prev(
        self, ret_type: type[_R2_co], start_time: float | datetime.datetime | None = None, update_current: bool = True
    ) -> _R2_co: ...
    @overload
    def get_prev(
        self, ret_type: None = None, start_time: float | datetime.datetime | None = None, update_current: bool = True
    ) -> _R_co: ...
    @overload
    def get_current(self, ret_type: type[_R2_co]) -> _R2_co: ...
    @overload
    def get_current(self, ret_type: None = None) -> _R_co: ...
    def set_current(self, start_time: float | datetime.datetime | None, force: bool = True) -> float: ...
    @staticmethod
    def datetime_to_timestamp(d: datetime.datetime) -> float: ...
    def timestamp_to_datetime(self, timestamp: float, tzinfo: datetime.tzinfo | None = ...) -> datetime.datetime: ...
    @staticmethod
    def timedelta_to_seconds(td: datetime.timedelta) -> float: ...
    @overload
    def all_next(
        self, ret_type: type[_R2_co], start_time: float | datetime.datetime | None = None, update_current: bool | None = None
    ) -> Generator[_R2_co]: ...
    @overload
    def all_next(
        self, ret_type: None = None, start_time: float | datetime.datetime | None = None, update_current: bool | None = None
    ) -> Generator[_R_co]: ...
    @overload
    def all_prev(
        self, ret_type: type[_R2_co], start_time: float | datetime.datetime | None = None, update_current: bool | None = None
    ) -> Generator[_R2_co]: ...
    @overload
    def all_prev(
        self, ret_type: None = None, start_time: float | datetime.datetime | None = None, update_current: bool | None = None
    ) -> Generator[_R_co]: ...
    def iter(self, *args: Unused, **kwargs: Unused) -> _AllIter[_R_co]: ...
    def __iter__(self) -> Self: ...
    @overload
    def next(
        self,
        ret_type: type[_R2_co],
        start_time: float | datetime.datetime | None = None,
        is_prev: bool | None = None,
        update_current: bool | None = None,
    ) -> _R2_co: ...
    @overload
    def next(
        self,
        ret_type: None = None,
        start_time: float | datetime.datetime | None = None,
        is_prev: bool | None = None,
        update_current: bool | None = None,
    ) -> _R_co: ...
    __next__ = next
    @staticmethod
    def is_leap(year: int) -> bool: ...
    @classmethod
    def value_alias(
        cls,
        val: int,
        field_index: Literal[0, 1, 2, 3, 4, 5, 6],
        len_expressions: int | list[Any] | dict[Any, Any] | tuple[Any, ...] | set[Any] = 5,
    ) -> int: ...
    @classmethod
    def expand(
        cls,
        expr_format: str,
        hash_id: bytes | None = None,
        second_at_beginning: bool = False,
        from_timestamp: float | None = None,
    ) -> tuple[list[list[str]], dict[str, set[int]]]: ...
    @classmethod
    def is_valid(
        cls, expression: str, hash_id: bytes | None = None, encoding: str = "UTF-8", second_at_beginning: bool = False
    ) -> bool: ...
    @classmethod
    def match(
        cls,
        cron_expression: str,
        testdate: float | datetime.datetime | None,
        day_or: bool = True,
        second_at_beginning: bool = False,
    ) -> bool: ...
    @classmethod
    def match_range(
        cls,
        cron_expression: str,
        from_datetime: datetime.datetime,
        to_datetime: datetime.datetime,
        day_or: bool = True,
        second_at_beginning: bool = False,
    ) -> bool: ...

@overload
def croniter_range(
    start: float | datetime.datetime,
    stop: float | datetime.datetime,
    expr_format: str,
    ret_type: type[_R2_co],
    day_or: bool = True,
    exclude_ends: bool = False,
    _croniter: type[croniter] | None = None,
    second_at_beginning: bool = False,
    expand_from_start_time: bool = False,
) -> Generator[_R2_co, None, None]: ...
@overload
def croniter_range(
    start: float,
    stop: float | datetime.datetime,
    expr_format: str,
    ret_type: None = None,
    day_or: bool = True,
    exclude_ends: bool = False,
    _croniter: type[croniter] | None = None,
    second_at_beginning: bool = False,
    expand_from_start_time: bool = False,
) -> Generator[float, None, None]: ...
@overload
def croniter_range(
    start: datetime.datetime,
    stop: float | datetime.datetime,
    expr_format: str,
    ret_type: None = None,
    day_or: bool = True,
    exclude_ends: bool = False,
    _croniter: type[croniter] | None = None,
    second_at_beginning: bool = False,
    expand_from_start_time: bool = False,
) -> Generator[datetime.datetime, None, None]: ...

class HashExpander:
    cron: croniter
    def __init__(self, cronit: croniter) -> None: ...
    @overload
    def do(
        self,
        idx: int,
        hash_type: Literal["r"],
        hash_id: None = None,
        range_end: int | None = None,
        range_begin: int | None = None,
    ) -> int: ...
    @overload
    def do(
        self, idx: int, hash_type: str, hash_id: bytes, range_end: int | None = None, range_begin: int | None = None
    ) -> int: ...
    @overload
    def do(
        self, idx: int, hash_type: str = "h", *, hash_id: bytes, range_end: int | None = None, range_begin: int | None = None
    ) -> int: ...
    def match(self, efl: Unused, idx: Unused, expr: str, hash_id: bytes | None = None, **kw: Unused) -> Match[str] | None: ...
    def expand(
        self,
        efl: object,
        idx: int,
        expr: str,
        hash_id: bytes | None = None,
        match: Match[str] | None | Literal[""] = "",
        **kw: object,
    ) -> str: ...

EXPANDERS: OrderedDict[str, HashExpander]



import datetime
from typing import ClassVar

from ...core import WesternCalendar
from .mixins import (
    AutumnHolidayFirstMondayOctober,
    AutumnHolidayLastMondaySeptember,
    AutumnHolidaySecondMondayOctober,
    AutumnHolidayThirdMondayOctober,
    AyrGoldCup,
    BattleStirlingBridge,
    FairHolidayFirstMondayAugust,
    FairHolidayFirstMondayJuly,
    FairHolidayFourthFridayJuly,
    FairHolidayLastMondayJuly,
    FairHolidayLastMondayJune,
    FairHolidaySecondMondayJuly,
    FairHolidayThirdMondayJuly,
    LateSummer,
    SpringHolidayFirstMondayApril,
    SpringHolidayFirstMondayJune,
    SpringHolidayLastMondayMay,
    SpringHolidaySecondMondayApril,
    SpringHolidayTuesdayAfterFirstMondayMay,
    VictoriaDayFirstMondayJune,
    VictoriaDayFourthMondayMay,
    VictoriaDayLastMondayMay,
)

class Scotland(WesternCalendar):
    include_spring_holiday: ClassVar[bool]
    spring_holiday_label: ClassVar[str]
    include_fair_holiday: ClassVar[bool]
    include_autumn_holiday: ClassVar[bool]
    include_saint_andrew: ClassVar[bool]
    include_victoria_day: ClassVar[bool]
    def __init__(self, *args, **kwargs) -> None: ...
    def get_may_day(self, year: int) -> tuple[datetime.date, str]: ...
    def get_spring_holiday(self, year: int) -> tuple[datetime.date, str]: ...
    def get_fair_holiday(self, year: int) -> tuple[datetime.date, str]: ...
    def get_autumn_holiday(self, year: int) -> tuple[datetime.date, str]: ...
    def get_victoria_day(self, year: int) -> tuple[datetime.date, str]: ...

class Aberdeen(FairHolidaySecondMondayJuly, AutumnHolidayLastMondaySeptember, Scotland): ...
class Angus(SpringHolidaySecondMondayApril, AutumnHolidayLastMondaySeptember, Scotland): ...
class Arbroath(FairHolidayThirdMondayJuly, Scotland): ...
class Ayr(SpringHolidayLastMondayMay, AyrGoldCup, Scotland): ...
class CarnoustieMonifieth(SpringHolidayFirstMondayApril, AutumnHolidayFirstMondayOctober, Scotland): ...
class Clydebank(SpringHolidayTuesdayAfterFirstMondayMay, Scotland): ...
class DumfriesGalloway(Scotland): ...
class Dundee(
    SpringHolidayFirstMondayApril, VictoriaDayLastMondayMay, FairHolidayLastMondayJuly, AutumnHolidayFirstMondayOctober, Scotland
): ...
class EastDunbartonshire(SpringHolidayLastMondayMay, FairHolidayThirdMondayJuly, AutumnHolidayLastMondaySeptember, Scotland): ...
class Edinburgh(Scotland): ...
class Elgin(SpringHolidaySecondMondayApril, FairHolidayLastMondayJune, LateSummer, AutumnHolidayThirdMondayOctober, Scotland): ...
class Falkirk(FairHolidayFirstMondayJuly, BattleStirlingBridge, Scotland): ...
class Fife(VictoriaDayFirstMondayJune, FairHolidayThirdMondayJuly, AutumnHolidayThirdMondayOctober, Scotland): ...
class Galashiels(SpringHolidayFirstMondayJune, Scotland): ...
class Glasgow(SpringHolidayLastMondayMay, FairHolidayThirdMondayJuly, AutumnHolidayLastMondaySeptember, Scotland): ...
class Hawick(Scotland): ...
class Inverclyde(LateSummer, Scotland): ...
class Inverness(SpringHolidayFirstMondayApril, FairHolidayFirstMondayJuly, AutumnHolidayFirstMondayOctober, Scotland): ...
class Kilmarnock(AyrGoldCup, Scotland): ...
class Lanark(Scotland): ...
class Linlithgow(Scotland): ...
class Lochaber(Scotland): ...
class NorthLanarkshire(SpringHolidayLastMondayMay, FairHolidayThirdMondayJuly, AutumnHolidayLastMondaySeptember, Scotland): ...
class Paisley(VictoriaDayLastMondayMay, FairHolidayFirstMondayAugust, AutumnHolidayLastMondaySeptember, Scotland): ...
class Perth(
    SpringHolidayFirstMondayApril, VictoriaDayFourthMondayMay, BattleStirlingBridge, AutumnHolidayFirstMondayOctober, Scotland
): ...
class ScottishBorders(SpringHolidayFirstMondayApril, FairHolidayFourthFridayJuly, AutumnHolidaySecondMondayOctober, Scotland): ...
class SouthLanarkshire(SpringHolidayLastMondayMay, FairHolidayThirdMondayJuly, AutumnHolidayLastMondaySeptember, Scotland): ...
class Stirling(SpringHolidayTuesdayAfterFirstMondayMay, BattleStirlingBridge, Scotland): ...
class WestDunbartonshire(AutumnHolidayLastMondaySeptember, Scotland): ...



from typing import ClassVar, Final

from docutils.transforms import Transform

__docformat__: Final = "reStructuredText"

class Filter(Transform):
    default_priority: ClassVar[int]
    def apply(self) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class ErrorSourceProto(google.protobuf.message.Message):
    """If included as a payload, this message contains the error source information
    where the error was raised.
    URI: "type.googleapis.com/tensorflow.core.platform.ErrorSourceProto"
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _ErrorSource:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _ErrorSourceEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ErrorSourceProto._ErrorSource.ValueType], builtins.type
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UNKNOWN: ErrorSourceProto._ErrorSource.ValueType  # 0
        TPU_COMPILE_OP: ErrorSourceProto._ErrorSource.ValueType  # 1
        TF_XLA_BRIDGE: ErrorSourceProto._ErrorSource.ValueType  # 2
        """Old bridge."""
        MLIR_BRIDGE_PHASE_1: ErrorSourceProto._ErrorSource.ValueType  # 3
        """TPUBridge."""
        MLIR_BRIDGE_PHASE_2: ErrorSourceProto._ErrorSource.ValueType  # 4
        """LegalizeToHlo."""
        EAGER_REMOTE_MGR: ErrorSourceProto._ErrorSource.ValueType  # 5
        """eager::RemoteMgr."""

    class ErrorSource(_ErrorSource, metaclass=_ErrorSourceEnumTypeWrapper): ...
    UNKNOWN: ErrorSourceProto.ErrorSource.ValueType  # 0
    TPU_COMPILE_OP: ErrorSourceProto.ErrorSource.ValueType  # 1
    TF_XLA_BRIDGE: ErrorSourceProto.ErrorSource.ValueType  # 2
    """Old bridge."""
    MLIR_BRIDGE_PHASE_1: ErrorSourceProto.ErrorSource.ValueType  # 3
    """TPUBridge."""
    MLIR_BRIDGE_PHASE_2: ErrorSourceProto.ErrorSource.ValueType  # 4
    """LegalizeToHlo."""
    EAGER_REMOTE_MGR: ErrorSourceProto.ErrorSource.ValueType  # 5
    """eager::RemoteMgr."""

    ERROR_SOURCE_FIELD_NUMBER: builtins.int
    error_source: global___ErrorSourceProto.ErrorSource.ValueType
    def __init__(self, *, error_source: global___ErrorSourceProto.ErrorSource.ValueType | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["error_source", b"error_source"]) -> None: ...

global___ErrorSourceProto = ErrorSourceProto



from __future__ import annotations

from typing import Protocol, cast
from typing_extensions import assert_type

import grpc.aio


class DummyRequest:
    pass


class DummyReply:
    pass


class DummyServiceStub(Protocol):
    UnaryUnary: grpc.aio.UnaryUnaryMultiCallable[DummyRequest, DummyReply]
    UnaryStream: grpc.aio.UnaryStreamMultiCallable[DummyRequest, DummyReply]
    StreamUnary: grpc.aio.StreamUnaryMultiCallable[DummyRequest, DummyReply]
    StreamStream: grpc.aio.StreamStreamMultiCallable[DummyRequest, DummyReply]


stub = cast(DummyServiceStub, None)
req = DummyRequest()


async def async_context() -> None:
    assert_type(await stub.UnaryUnary(req), DummyReply)

    async for resp in stub.UnaryStream(req):
        assert_type(resp, DummyReply)

    assert_type(await stub.StreamUnary(iter([req])), DummyReply)

    async for resp in stub.StreamStream(iter([req])):
        assert_type(resp, DummyReply)



__all__ = ["expand_des_key", "des_encrypt_block"]

def expand_des_key(key): ...
def des_encrypt_block(key, input, salt: int = 0, rounds: int = 1): ...
def des_encrypt_int_block(key, input, salt: int = 0, rounds: int = 1): ...



from _typeshed import Incomplete
from logging import Logger
from typing import Literal

from oauthlib.common import Request, _HTTPMethod

from .base import BaseEndpoint

log: Logger

class TokenEndpoint(BaseEndpoint):
    valid_request_methods: tuple[Literal["POST"]]
    def __init__(self, default_grant_type: str, default_token_type: str, grant_types: dict[str, Incomplete]) -> None: ...
    @property
    def grant_types(self) -> dict[str, Incomplete]: ...
    @property
    def default_grant_type(self) -> str: ...
    @property
    def default_grant_type_handler(self): ...
    @property
    def default_token_type(self) -> str: ...
    def create_token_response(
        self,
        uri: str,
        http_method: _HTTPMethod = "POST",
        body: str | dict[str, str] | list[tuple[str, str]] | None = None,
        headers: dict[str, str] | None = None,
        credentials=None,
        grant_type_for_scope=None,
        claims=None,
    ): ...
    def validate_token_request(self, request: Request) -> None: ...



from collections.abc import Generator, Sequence
from logging import Logger

LOG: Logger

def expand_paths(
    *, paths: Sequence[str], stdin_display_name: str, filename_patterns: Sequence[str], exclude: Sequence[str]
) -> Generator[str, None, None]: ...



from _typeshed import StrPath, SupportsRead, SupportsWrite
from typing import IO, Literal, Protocol, type_check_only
from typing_extensions import TypeAlias

from openpyxl.compat.numbers import NUMPY as NUMPY
from openpyxl.reader.excel import load_workbook as load_workbook
from openpyxl.workbook import Workbook as Workbook
from openpyxl.xml import DEFUSEDXML as DEFUSEDXML, LXML as LXML

from ._constants import (
    __author__ as __author__,
    __author_email__ as __author_email__,
    __license__ as __license__,
    __maintainer_email__ as __maintainer_email__,
    __url__ as __url__,
    __version__ as __version__,
)

DEBUG: bool
open = load_workbook

# Utility types reused elsewhere
_VisibilityType: TypeAlias = Literal["visible", "hidden", "veryHidden"]  # noqa: Y047

# TODO: Use a proper protocol from ZipFile. See: #10880
# This alias is to minimize false-positives
_ZipFileFileProtocol: TypeAlias = StrPath | IO[bytes] | SupportsRead[bytes]  # noqa: Y047
_ZipFileFileWriteProtocol: TypeAlias = StrPath | IO[bytes] | SupportsWrite[bytes]  # noqa: Y047

@type_check_only
class _Decodable(Protocol):  # noqa: Y046
    def decode(self, encoding: str, /) -> str: ...



import gzip
from _typeshed import ReadableBuffer
from typing import Final, Protocol, type_check_only
from xmlrpc.client import ExpatParser, Unmarshaller

@type_check_only
class _Readable(Protocol):
    def read(self, size: int | None = -1) -> bytes: ...

__origin__: Final = "xmlrpc.client"
MAX_DATA: Final = 31457280

def defused_gzip_decode(data: ReadableBuffer, limit: int | None = None) -> bytes: ...

class DefusedGzipDecodedResponse(gzip.GzipFile):
    limit: int
    readlength: int | None
    def __init__(self, response: _Readable, limit: int | None = None) -> None: ...
    def read(self, n: int) -> bytes: ...  # type: ignore[override]
    def close(self) -> None: ...

class DefusedExpatParser(ExpatParser):
    forbid_dtd: bool
    forbid_entities: bool
    forbid_external: bool
    def __init__(
        self, target: Unmarshaller, forbid_dtd: bool = False, forbid_entities: bool = True, forbid_external: bool = True
    ) -> None: ...
    def defused_start_doctype_decl(self, name: str, sysid: str | None, pubid: str | None, has_internal_subset: bool) -> None: ...
    def defused_entity_decl(
        self,
        name: str,
        is_parameter_entity: bool,
        value: str | None,
        base: str | None,
        sysid: str,
        pubid: str | None,
        notation_name: str | None,
    ) -> None: ...
    def defused_unparsed_entity_decl(
        self, name: str, base: str | None, sysid: str, pubid: str | None, notation_name: str
    ) -> None: ...
    def defused_external_entity_ref_handler(
        self, context: str, base: str | None, sysid: str | None, pubid: str | None
    ) -> None: ...

def monkey_patch() -> None: ...
def unmonkey_patch() -> None: ...



from typing import Final

__version__: Final[str]
PLAIN: int
PREFORMATTED: int
BULLETCHAR: str

class BaseParser:
    def __init__(self) -> None: ...
    def reset(self) -> None: ...
    def parseFile(self, filename): ...
    def parseText(self, textBlock): ...
    def readLine(self, line) -> None: ...
    def endPara(self) -> None: ...
    def beginPre(self, stylename) -> None: ...
    def endPre(self) -> None: ...
    def image(self, filename) -> None: ...

class Parser(BaseParser):
    def vSpace(self, points) -> None: ...
    def pageBreak(self) -> None: ...
    def custom(self, moduleName, funcName) -> None: ...
    def nextPageTemplate(self, templateName) -> None: ...

def parseFile(filename): ...
def parseText(textBlock): ...



from _typeshed import Incomplete, SupportsGetItem
from collections.abc import Callable, Collection, Generator, Iterable
from typing import Any

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["all_simple_paths", "is_simple_path", "shortest_simple_paths", "all_simple_edge_paths"]

@_dispatchable
def is_simple_path(G: Graph[_Node], nodes: Collection[Incomplete]) -> bool: ...
@_dispatchable
def all_simple_paths(
    G: Graph[_Node], source: _Node, target: _Node | Iterable[_Node], cutoff: int | None = None
) -> Generator[list[_Node]]: ...
@_dispatchable
def all_simple_edge_paths(
    G: Graph[_Node], source: _Node, target: _Node | Iterable[_Node], cutoff: int | None = None
) -> Generator[list[_Node] | list[tuple[_Node, _Node]], None, list[_Node] | None]: ...
@_dispatchable
def shortest_simple_paths(
    G: Graph[_Node],
    source: _Node,
    target: _Node,
    weight: str | Callable[[Any, Any, SupportsGetItem[str, Any]], float | None] | None = None,
) -> Generator[list[_Node], None, None]: ...

class PathBuffer:
    paths: Incomplete
    sortedpaths: Incomplete
    counter: Incomplete

    def __init__(self) -> None: ...
    def __len__(self): ...
    def push(self, cost, path) -> None: ...
    def pop(self): ...



import optparse
from logging import Logger
from typing import Any

logger: Logger

def parse_options(
    args: list[str] | None = None, values: optparse.Values | None = None
) -> tuple[dict[str, Any], Any]: ...  # first item is opts dict, second item is Values.verbose field
def run() -> None: ...



from .base import AuthenticationBase

class GetToken(AuthenticationBase):
    def authorization_code(self, code: str, redirect_uri: str | None, grant_type: str = "authorization_code"): ...
    def authorization_code_pkce(
        self, code_verifier: str, code: str, redirect_uri: str | None, grant_type: str = "authorization_code"
    ): ...
    def client_credentials(self, audience: str, grant_type: str = "client_credentials", organization: str | None = None): ...
    def login(
        self,
        username: str,
        password: str,
        scope: str | None = None,
        realm: str | None = None,
        audience: str | None = None,
        grant_type: str = "http://auth0.com/oauth/grant-type/password-realm",
        forwarded_for: str | None = None,
    ): ...
    def refresh_token(self, refresh_token: str, scope: str = "", grant_type: str = "refresh_token"): ...
    def passwordless_login(self, username: str, otp: str, realm: str, scope: str, audience: str): ...
    def backchannel_login(self, auth_req_id: str, grant_type: str = "urn:openid:params:grant-type:ciba"): ...
    def access_token_for_connection(
        self,
        subject_token_type: str,
        subject_token: str,
        requested_token_type: str,
        connection: str | None = None,
        grant_type: str = ...,
    ): ...



from typing import Final

from pymeeus.Angle import Angle
from pymeeus.Epoch import Epoch

VSOP87_L: Final[list[list[list[float]]]]
VSOP87_B: Final[list[list[list[float]]]]
VSOP87_R: Final[list[list[list[float]]]]
VSOP87_L_J2000: Final[list[list[list[float]]]]
VSOP87_B_J2000: Final[list[list[list[float]]]]
ORBITAL_ELEM: Final[list[list[float]]]
ORBITAL_ELEM_J2000: Final[list[list[float]]]

class Ellipsoid:
    def __init__(self, a: float, f: float, omega: float) -> None: ...
    def b(self) -> float: ...
    def e(self) -> float: ...

IAU76: Final[Ellipsoid]
WGS84: Final[Ellipsoid]

class Earth:
    def __init__(self, ellipsoid: Ellipsoid = ...) -> None: ...
    def set(self, ellipsoid: Ellipsoid) -> None: ...
    def rho(self, latitude: float | Angle) -> float: ...
    def rho_sinphi(self, latitude: float | Angle, height: float) -> float: ...
    def rho_cosphi(self, latitude: float | Angle, height: float) -> float: ...
    def rp(self, latitude: float | Angle) -> float: ...
    def linear_velocity(self, latitude: float | Angle) -> float: ...
    def rm(self, latitude: float | Angle) -> float: ...
    def distance(
        self, lon1: float | Angle, lat1: float | Angle, lon2: float | Angle, lat2: float | Angle
    ) -> tuple[float, float]: ...
    @staticmethod
    def geometric_heliocentric_position(epoch: Epoch, tofk5: bool = True) -> tuple[Angle, Angle, float]: ...
    @staticmethod
    def apparent_heliocentric_position(epoch: Epoch, nutation: bool = True) -> tuple[Angle, Angle, float]: ...
    @staticmethod
    def geometric_heliocentric_position_j2000(epoch: Epoch, tofk5: bool = True) -> tuple[Angle, Angle, float]: ...
    @staticmethod
    def orbital_elements_mean_equinox(epoch: Epoch) -> tuple[Angle, float, float, Angle, Angle, Angle]: ...
    @staticmethod
    def orbital_elements_j2000(epoch: Epoch) -> tuple[Angle, float, float, Angle, Angle, Angle]: ...
    @staticmethod
    def perihelion_aphelion(epoch: Epoch, perihelion: bool | None = True) -> Epoch: ...
    @staticmethod
    def passage_nodes(epoch: Epoch, ascending: bool = True) -> tuple[Epoch, float]: ...
    @staticmethod
    def parallax_correction(
        right_ascension: Angle, declination: Angle, latitude: Angle, distance: float, hour_angle: Angle, height: float = 0.0
    ) -> tuple[Angle, Angle]: ...
    @staticmethod
    def parallax_ecliptical(
        longitude: Angle,
        latitude: Angle,
        semidiameter: Angle,
        obs_lat: Angle,
        obliquity: Angle,
        sidereal_time: Angle,
        distance: float,
        height: float = 0.0,
    ) -> tuple[Angle, Angle, Angle]: ...

def main() -> None: ...



from authlib.oauth2 import OAuth2Error

class InvalidRedirectURIError(OAuth2Error):
    error: str

class InvalidClientMetadataError(OAuth2Error):
    error: str

class InvalidSoftwareStatementError(OAuth2Error):
    error: str

class UnapprovedSoftwareStatementError(OAuth2Error):
    error: str



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Copyright 2023 The OpenXLA Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
==============================================================================
"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.duration_pb2
import google.protobuf.internal.containers
import google.protobuf.message
import tensorflow.compiler.xla.service.hlo_pb2
import tensorflow.tsl.protobuf.status_pb2

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class CompilerPerfStats(google.protobuf.message.Message):
    """Statistics on how long various parts of compilation took.
    Not all durations may be relevant for all producers of this message, in
    which irrelevant fields should simply be skipped.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    INIT_DURATION_FIELD_NUMBER: builtins.int
    HLO_VERIFICATION_DURATION_FIELD_NUMBER: builtins.int
    COMPILATION_PROLOGUE_DURATION_FIELD_NUMBER: builtins.int
    COMPILATION_DURATION_FIELD_NUMBER: builtins.int
    TOTAL_DURATION_FIELD_NUMBER: builtins.int
    @property
    def init_duration(self) -> google.protobuf.duration_pb2.Duration:
        """How long did it take to initialize the compiler?"""

    @property
    def hlo_verification_duration(self) -> google.protobuf.duration_pb2.Duration:
        """How long did it take to verify the HLO?"""

    @property
    def compilation_prologue_duration(self) -> google.protobuf.duration_pb2.Duration:
        """How long did it take to prepare for compilation after verification?"""

    @property
    def compilation_duration(self) -> google.protobuf.duration_pb2.Duration:
        """How long did it take to compile?"""

    @property
    def total_duration(self) -> google.protobuf.duration_pb2.Duration:
        """How long did everything take?"""

    def __init__(
        self,
        *,
        init_duration: google.protobuf.duration_pb2.Duration | None = ...,
        hlo_verification_duration: google.protobuf.duration_pb2.Duration | None = ...,
        compilation_prologue_duration: google.protobuf.duration_pb2.Duration | None = ...,
        compilation_duration: google.protobuf.duration_pb2.Duration | None = ...,
        total_duration: google.protobuf.duration_pb2.Duration | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "compilation_duration",
            b"compilation_duration",
            "compilation_prologue_duration",
            b"compilation_prologue_duration",
            "hlo_verification_duration",
            b"hlo_verification_duration",
            "init_duration",
            b"init_duration",
            "total_duration",
            b"total_duration",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "compilation_duration",
            b"compilation_duration",
            "compilation_prologue_duration",
            b"compilation_prologue_duration",
            "hlo_verification_duration",
            b"hlo_verification_duration",
            "init_duration",
            b"init_duration",
            "total_duration",
            b"total_duration",
        ],
    ) -> None: ...

global___CompilerPerfStats = CompilerPerfStats

@typing.final
class CompilationResult(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class CountersEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.int
        def __init__(self, *, key: builtins.str | None = ..., value: builtins.int | None = ...) -> None: ...
        def HasField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    HLO_MODULE_FIELD_NUMBER: builtins.int
    PERF_STATS_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    COUNTERS_FIELD_NUMBER: builtins.int
    @property
    def hlo_module(self) -> tensorflow.compiler.xla.service.hlo_pb2.HloModuleProto:
        """The compiled HLO. Only set when compilation succeeds."""

    @property
    def perf_stats(self) -> global___CompilerPerfStats:
        """Always set when compilation succeeds. May or may not be set when
        compilation fails.
        """

    @property
    def status(self) -> tensorflow.tsl.protobuf.status_pb2.StatusProto:
        """Always set even when compilation succeeds."""

    @property
    def counters(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.int]:
        """Collects counters collected during compilation. Not every producer may
        include counter support at all or any particular counter.
        """

    def __init__(
        self,
        *,
        hlo_module: tensorflow.compiler.xla.service.hlo_pb2.HloModuleProto | None = ...,
        perf_stats: global___CompilerPerfStats | None = ...,
        status: tensorflow.tsl.protobuf.status_pb2.StatusProto | None = ...,
        counters: collections.abc.Mapping[builtins.str, builtins.int] | None = ...,
    ) -> None: ...
    def HasField(
        self, field_name: typing.Literal["hlo_module", b"hlo_module", "perf_stats", b"perf_stats", "status", b"status"]
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "counters", b"counters", "hlo_module", b"hlo_module", "perf_stats", b"perf_stats", "status", b"status"
        ],
    ) -> None: ...

global___CompilationResult = CompilationResult



import enum
import io
import queue
import threading
from collections.abc import Callable, Iterable
from typing import Any, ClassVar, TypeVar
from typing_extensions import TypeAlias

import gdb

_T = TypeVar("_T")
_F = TypeVar("_F", bound=Callable[..., Any])

DAPQueue: TypeAlias = queue.SimpleQueue[_T]

class DAPException(Exception): ...

def parse_and_eval(expression: str, global_context: bool = False) -> gdb.Value: ...

# target and args are passed to gdb.Thread
def start_thread(name: str, target: Callable[..., object], args: Iterable[Any] = ()) -> gdb.Thread: ...
def start_dap(target: Callable[..., object]) -> None: ...
def in_gdb_thread(func: _F) -> _F: ...
def in_dap_thread(func: _F) -> _F: ...

class LogLevel(enum.IntEnum):
    DEFAULT = 1
    FULL = 2

class LogLevelParam(gdb.Parameter):
    def __init__(self) -> None: ...

class LoggingParam(gdb.Parameter):
    lock: ClassVar[threading.Lock]
    log_file: io.TextIOWrapper | None
    def __init__(self) -> None: ...
    def get_set_string(self) -> str: ...

dap_log: LoggingParam

def log(something: object, level: LogLevel = LogLevel.DEFAULT) -> None: ...
def thread_log(something: object, level: LogLevel = LogLevel.DEFAULT) -> None: ...
def log_stack(level: LogLevel = LogLevel.DEFAULT) -> None: ...
def exec_and_log(cmd: str) -> None: ...



from _typeshed import Incomplete, Unused
from abc import ABCMeta, abstractmethod
from collections.abc import Callable

from pyasn1.type import base, char, univ, useful
from pyasn1.type.base import Asn1Type
from pyasn1.type.tag import TagSet

__all__ = ["StreamingDecoder", "Decoder", "decode"]

class AbstractPayloadDecoder:
    protoComponent: Asn1Type | None
    @abstractmethod
    def valueDecoder(
        self,
        substrate,
        asn1Spec,
        tagSet: TagSet | None = None,
        length: int | None = None,
        state=None,
        decodeFun: Callable[..., Incomplete] | None = None,
        substrateFun: Callable[..., Incomplete] | None = None,
        **options,
    ) -> None: ...
    # Abstract, but implementation is optional
    def indefLenValueDecoder(
        self,
        substrate,
        asn1Spec,
        tagSet: TagSet | None = None,
        length: int | None = None,
        state=None,
        decodeFun: Callable[..., Incomplete] | None = None,
        substrateFun: Callable[..., Incomplete] | None = None,
        **options,
    ) -> None: ...

class AbstractSimplePayloadDecoder(AbstractPayloadDecoder, metaclass=ABCMeta):
    @staticmethod
    def substrateCollector(asn1Object, substrate, length, options): ...

class RawPayloadDecoder(AbstractSimplePayloadDecoder):
    protoComponent: univ.Any
    def valueDecoder(
        self,
        substrate,
        asn1Spec,
        tagSet: TagSet | None = None,
        length: int | None = None,
        state: Unused = None,
        decodeFun: Callable[..., Incomplete] | None = None,
        substrateFun: Callable[..., Incomplete] | None = None,
        **options,
    ): ...
    def indefLenValueDecoder(
        self,
        substrate,
        asn1Spec,
        tagSet: TagSet | None = None,
        length: int | None = None,
        state: Unused = None,
        decodeFun: Callable[..., Incomplete] | None = None,
        substrateFun: Callable[..., Incomplete] | None = None,
        **options,
    ): ...

class IntegerPayloadDecoder(AbstractSimplePayloadDecoder):
    protoComponent: univ.Integer
    def valueDecoder(
        self,
        substrate,
        asn1Spec,
        tagSet: TagSet | None = None,
        length: int | None = None,
        state: Unused = None,
        decodeFun: Unused = None,
        substrateFun: Unused = None,
        **options,
    ): ...

class BooleanPayloadDecoder(IntegerPayloadDecoder):
    protoComponent: univ.Boolean

class BitStringPayloadDecoder(AbstractSimplePayloadDecoder):
    protoComponent: univ.BitString
    supportConstructedForm: bool
    def valueDecoder(
        self,
        substrate,
        asn1Spec,
        tagSet: TagSet | None = None,
        length: int | None = None,
        state: Unused = None,
        decodeFun: Callable[..., Incomplete] | None = None,
        substrateFun: Callable[..., Incomplete] | None = None,
        **options,
    ): ...
    def indefLenValueDecoder(
        self,
        substrate,
        asn1Spec,
        tagSet: TagSet | None = None,
        length: int | None = None,
        state: Unused = None,
        decodeFun: Callable[..., Incomplete] | None = None,
        substrateFun: Callable[..., Incomplete] | None = None,
        **options,
    ): ...

class OctetStringPayloadDecoder(AbstractSimplePayloadDecoder):
    protoComponent: univ.OctetString
    supportConstructedForm: bool
    def valueDecoder(
        self,
        substrate,
        asn1Spec,
        tagSet: TagSet | None = None,
        length: int | None = None,
        state: Unused = None,
        decodeFun: Callable[..., Incomplete] | None = None,
        substrateFun: Callable[..., Incomplete] | None = None,
        **options,
    ): ...
    def indefLenValueDecoder(
        self,
        substrate,
        asn1Spec,
        tagSet: TagSet | None = None,
        length: int | None = None,
        state: Unused = None,
        decodeFun: Callable[..., Incomplete] | None = None,
        substrateFun: Callable[..., Incomplete] | None = None,
        **options,
    ): ...

class NullPayloadDecoder(AbstractSimplePayloadDecoder):
    protoComponent: univ.Null
    def valueDecoder(
        self,
        substrate,
        asn1Spec,
        tagSet: TagSet | None = None,
        length: int | None = None,
        state: Unused = None,
        decodeFun: Unused = None,
        substrateFun: Unused = None,
        **options,
    ): ...

class ObjectIdentifierPayloadDecoder(AbstractSimplePayloadDecoder):
    protoComponent: univ.ObjectIdentifier
    def valueDecoder(
        self,
        substrate,
        asn1Spec,
        tagSet: TagSet | None = None,
        length: int | None = None,
        state: Unused = None,
        decodeFun: Unused = None,
        substrateFun: Unused = None,
        **options,
    ): ...

class RealPayloadDecoder(AbstractSimplePayloadDecoder):
    protoComponent: univ.Real
    def valueDecoder(
        self,
        substrate,
        asn1Spec,
        tagSet: TagSet | None = None,
        length: int | None = None,
        state: Unused = None,
        decodeFun: Unused = None,
        substrateFun: Unused = None,
        **options,
    ): ...

class AbstractConstructedPayloadDecoder(AbstractPayloadDecoder, metaclass=ABCMeta):
    protoComponent: base.ConstructedAsn1Type | None

class ConstructedPayloadDecoderBase(AbstractConstructedPayloadDecoder):
    protoRecordComponent: univ.SequenceAndSetBase | None
    protoSequenceComponent: univ.SequenceOfAndSetOfBase | None
    def valueDecoder(
        self,
        substrate,
        asn1Spec,
        tagSet: TagSet | None = None,
        length: int | None = None,
        state: Unused = None,
        decodeFun: Callable[..., Incomplete] | None = None,
        substrateFun: Callable[..., Incomplete] | None = None,
        **options,
    ): ...
    def indefLenValueDecoder(
        self,
        substrate,
        asn1Spec,
        tagSet: TagSet | None = None,
        length: int | None = None,
        state: Unused = None,
        decodeFun: Callable[..., Incomplete] | None = None,
        substrateFun: Callable[..., Incomplete] | None = None,
        **options,
    ): ...

class SequenceOrSequenceOfPayloadDecoder(ConstructedPayloadDecoderBase):
    protoRecordComponent: univ.Sequence
    protoSequenceComponent: univ.SequenceOf

class SequencePayloadDecoder(SequenceOrSequenceOfPayloadDecoder):
    protoComponent: univ.Sequence

class SequenceOfPayloadDecoder(SequenceOrSequenceOfPayloadDecoder):
    protoComponent: univ.SequenceOf

class SetOrSetOfPayloadDecoder(ConstructedPayloadDecoderBase):
    protoRecordComponent: univ.Set
    protoSequenceComponent: univ.SetOf

class SetPayloadDecoder(SetOrSetOfPayloadDecoder):
    protoComponent: univ.Set

class SetOfPayloadDecoder(SetOrSetOfPayloadDecoder):
    protoComponent: univ.SetOf

class ChoicePayloadDecoder(AbstractConstructedPayloadDecoder):
    protoComponent: univ.Choice
    def valueDecoder(
        self,
        substrate,
        asn1Spec,
        tagSet: TagSet | None = None,
        length: int | None = None,
        state=None,
        decodeFun: Callable[..., Incomplete] | None = None,
        substrateFun: Callable[..., Incomplete] | None = None,
        **options,
    ): ...
    def indefLenValueDecoder(
        self,
        substrate,
        asn1Spec,
        tagSet: TagSet | None = None,
        length: int | None = None,
        state=None,
        decodeFun: Callable[..., Incomplete] | None = None,
        substrateFun: Callable[..., Incomplete] | None = None,
        **options,
    ): ...

class AnyPayloadDecoder(AbstractSimplePayloadDecoder):
    protoComponent: univ.Any
    def valueDecoder(
        self,
        substrate,
        asn1Spec,
        tagSet: TagSet | None = None,
        length: int | None = None,
        state: Unused = None,
        decodeFun: Unused = None,
        substrateFun: Callable[..., Incomplete] | None = None,
        **options,
    ): ...
    def indefLenValueDecoder(
        self,
        substrate,
        asn1Spec,
        tagSet: TagSet | None = None,
        length: int | None = None,
        state: Unused = None,
        decodeFun: Callable[..., Incomplete] | None = None,
        substrateFun: Callable[..., Incomplete] | None = None,
        **options,
    ): ...

class UTF8StringPayloadDecoder(OctetStringPayloadDecoder):
    protoComponent: char.UTF8String

class NumericStringPayloadDecoder(OctetStringPayloadDecoder):
    protoComponent: char.NumericString

class PrintableStringPayloadDecoder(OctetStringPayloadDecoder):
    protoComponent: char.PrintableString

class TeletexStringPayloadDecoder(OctetStringPayloadDecoder):
    protoComponent: char.TeletexString

class VideotexStringPayloadDecoder(OctetStringPayloadDecoder):
    protoComponent: char.VideotexString

class IA5StringPayloadDecoder(OctetStringPayloadDecoder):
    protoComponent: char.IA5String

class GraphicStringPayloadDecoder(OctetStringPayloadDecoder):
    protoComponent: char.GraphicString

class VisibleStringPayloadDecoder(OctetStringPayloadDecoder):
    protoComponent: char.VisibleString

class GeneralStringPayloadDecoder(OctetStringPayloadDecoder):
    protoComponent: char.GeneralString

class UniversalStringPayloadDecoder(OctetStringPayloadDecoder):
    protoComponent: char.UniversalString

class BMPStringPayloadDecoder(OctetStringPayloadDecoder):
    protoComponent: char.BMPString

class ObjectDescriptorPayloadDecoder(OctetStringPayloadDecoder):
    protoComponent: useful.ObjectDescriptor

class GeneralizedTimePayloadDecoder(OctetStringPayloadDecoder):
    protoComponent: useful.GeneralizedTime

class UTCTimePayloadDecoder(OctetStringPayloadDecoder):
    protoComponent: useful.UTCTime

TAG_MAP: dict[TagSet, AbstractPayloadDecoder]
TYPE_MAP: dict[int, AbstractPayloadDecoder]
# deprecated aliases
tagMap = TAG_MAP
typeMap = TYPE_MAP

class SingleItemDecoder:
    defaultErrorState: int
    defaultRawDecoder: AnyPayloadDecoder
    supportIndefLength: bool
    TAG_MAP: dict[TagSet, AbstractPayloadDecoder]
    TYPE_MAP: dict[int, AbstractPayloadDecoder]
    def __init__(self, tagMap=..., typeMap=..., **ignored: Unused) -> None: ...
    def __call__(
        self,
        substrate,
        asn1Spec: Asn1Type | None = None,
        tagSet: TagSet | None = None,
        length: int | None = None,
        state=0,
        decodeFun: Unused = None,
        substrateFun: Callable[..., Incomplete] | None = None,
        **options,
    ): ...

decode: Decoder

class StreamingDecoder:
    SINGLE_ITEM_DECODER: type[SingleItemDecoder]

    def __init__(self, substrate, asn1Spec=None, *, tagMap=..., typeMap=..., **ignored: Unused) -> None: ...
    def __iter__(self): ...

class Decoder:
    STREAMING_DECODER: type[StreamingDecoder]

    @classmethod
    def __call__(cls, substrate, asn1Spec=None, *, tagMap=..., typeMap=..., **ignored: Unused): ...



from _typeshed import FileDescriptorOrPath, SupportsKeysAndGetItem, Unused
from collections.abc import Iterable
from struct import Struct
from typing import Final, Literal
from typing_extensions import Self, TypeAlias

from .bks import BksKeyEntry
from .jks import PrivateKeyEntry

b8: Final[Struct]
b4: Final[Struct]
b2: Final[Struct]
b1: Final[Struct]
py23basestring: Final[tuple[type[str], type[str]]]
RSA_ENCRYPTION_OID: Final[tuple[int, ...]]
DSA_OID: Final[tuple[int, ...]]
DSA_WITH_SHA1_OID: Final[tuple[int, ...]]

_KeystoreType: TypeAlias = Literal["jks", "jceks", "bks", "uber"]
_PemType: TypeAlias = Literal["CERTIFICATE", "PUBLIC KEY", "PRIVATE KEY", "RSA PRIVATE KEY"]

class KeystoreException(Exception): ...
class KeystoreSignatureException(KeystoreException): ...
class DuplicateAliasException(KeystoreException): ...
class NotYetDecryptedException(KeystoreException): ...
class BadKeystoreFormatException(KeystoreException): ...
class BadDataLengthException(KeystoreException): ...
class BadPaddingException(KeystoreException): ...
class BadHashCheckException(KeystoreException): ...
class DecryptionFailureException(KeystoreException): ...
class UnsupportedKeystoreVersionException(KeystoreException): ...
class UnexpectedJavaTypeException(KeystoreException): ...
class UnexpectedAlgorithmException(KeystoreException): ...
class UnexpectedKeyEncodingException(KeystoreException): ...
class UnsupportedKeystoreTypeException(KeystoreException): ...
class UnsupportedKeystoreEntryTypeException(KeystoreException): ...
class UnsupportedKeyFormatException(KeystoreException): ...

class AbstractKeystore:
    store_type: _KeystoreType
    entries: dict[str, AbstractKeystoreEntry]
    def __init__(self, store_type: _KeystoreType, entries: SupportsKeysAndGetItem[str, AbstractKeystoreEntry]) -> None: ...
    @classmethod
    def load(cls, filename: FileDescriptorOrPath, store_password: str | None, try_decrypt_keys: bool = True) -> Self: ...
    def save(self, filename: FileDescriptorOrPath, store_password: str) -> None: ...

class AbstractKeystoreEntry:
    store_type: _KeystoreType | None
    alias: str
    timestamp: int
    def __init__(self, *, store_type: _KeystoreType | None = None, alias: str, timestamp: int, **kwargs: Unused) -> None: ...
    @classmethod
    def new(cls, alias: str) -> Self: ...
    def is_decrypted(self) -> bool: ...
    def decrypt(self, key_password: str) -> None: ...
    def encrypt(self, key_password: str) -> None: ...

def as_hex(ba: bytes | bytearray) -> str: ...
def as_pem(der_bytes: bytes, type: _PemType) -> str: ...
def bitstring_to_bytes(bitstr: Iterable[int]) -> bytes: ...
def xor_bytearrays(a: bytes | bytearray, b: bytes | bytearray) -> bytearray: ...
def print_pem(der_bytes: bytes, type: _PemType) -> None: ...
def pkey_as_pem(pk: PrivateKeyEntry | BksKeyEntry) -> str: ...
def strip_pkcs5_padding(m: bytes | bytearray) -> bytes: ...
def strip_pkcs7_padding(m: bytes | bytearray, block_size: int) -> bytes: ...
def add_pkcs7_padding(m: bytes | bytearray, block_size: int) -> bytes: ...



"""Preconfigured converters for orjson."""

from cattrs.preconf.orjson import OrjsonConverter, configure_converter, make_converter

__all__ = ["configure_converter", "make_converter", "OrjsonConverter"]



from _typeshed import Incomplete

from ..rest import RestClientOptions
from ..types import TimeoutType

class AttackProtection:
    domain: Incomplete
    protocol: Incomplete
    client: Incomplete
    def __init__(
        self,
        domain: str,
        token: str,
        telemetry: bool = True,
        timeout: TimeoutType = 5.0,
        protocol: str = "https",
        rest_options: RestClientOptions | None = None,
    ) -> None: ...
    def get_breached_password_detection(self) -> dict[str, Incomplete]: ...
    async def get_breached_password_detection_async(self) -> dict[str, Incomplete]: ...
    def update_breached_password_detection(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    async def update_breached_password_detection_async(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    def get_brute_force_protection(self) -> dict[str, Incomplete]: ...
    async def get_brute_force_protection_async(self) -> dict[str, Incomplete]: ...
    def update_brute_force_protection(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    async def update_brute_force_protection_async(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    def get_suspicious_ip_throttling(self) -> dict[str, Incomplete]: ...
    async def get_suspicious_ip_throttling_async(self) -> dict[str, Incomplete]: ...
    def update_suspicious_ip_throttling(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    async def update_suspicious_ip_throttling_async(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...



from _typeshed import SupportsGetItem, SupportsKeysAndGetItem
from _typeshed.wsgi import WSGIEnvironment
from collections.abc import Collection, Iterable, Iterator, MutableMapping
from typing import Literal, Protocol, TypeVar, overload, type_check_only
from typing_extensions import Self

from webob.compat import cgi_FieldStorage, cgi_FieldStorage as _FieldStorageWithFile

__all__ = ["MultiDict", "NestedMultiDict", "NoVars", "GetDict"]

_T = TypeVar("_T")
_KT = TypeVar("_KT")
_VT = TypeVar("_VT")
_KT_co = TypeVar("_KT_co", covariant=True)
_VT_co = TypeVar("_VT_co", covariant=True)

@type_check_only
class _SupportsItemsWithIterableResult(Protocol[_KT_co, _VT_co]):
    def items(self) -> Iterable[tuple[_KT_co, _VT_co]]: ...

class MultiDict(MutableMapping[_KT, _VT]):
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self: MultiDict[str, _VT], **kwargs: _VT) -> None: ...  # pyright: ignore[reportInvalidTypeVarUse]  #11780
    @overload
    def __init__(self, m: _SupportsItemsWithIterableResult[_KT, _VT], /) -> None: ...
    @overload
    def __init__(
        self: MultiDict[str, _VT],  # pyright: ignore[reportInvalidTypeVarUse]  #11780
        m: _SupportsItemsWithIterableResult[str, _VT],
        /,
        **kwargs: _VT,
    ) -> None: ...
    @overload
    def __init__(self, m: Iterable[tuple[_KT, _VT]], /) -> None: ...
    @overload
    def __init__(
        self: MultiDict[str, _VT],  # pyright: ignore[reportInvalidTypeVarUse]  #11780
        m: Iterable[tuple[str, _VT]],
        /,
        **kwargs: _VT,
    ) -> None: ...
    @classmethod
    def view_list(cls, lst: list[tuple[_KT, _VT]]) -> Self: ...
    @classmethod
    def from_fieldstorage(cls, fs: cgi_FieldStorage) -> MultiDict[str, str | _FieldStorageWithFile]: ...
    def __getitem__(self, key: _KT) -> _VT: ...
    def __setitem__(self, key: _KT, value: _VT) -> None: ...
    def add(self, key: _KT, value: _VT) -> None: ...
    @overload
    def get(self, key: _KT, default: None = None) -> _VT | None: ...
    @overload
    def get(self, key: _KT, default: _VT) -> _VT: ...
    @overload
    def get(self, key: _KT, default: _T) -> _VT | _T: ...
    def getall(self, key: _KT) -> list[_VT]: ...
    def getone(self, key: _KT) -> _VT: ...
    def mixed(self) -> dict[_KT, _VT | list[_VT]]: ...
    def dict_of_lists(self) -> dict[_KT, list[_VT]]: ...
    def __delitem__(self, key: _KT) -> None: ...
    def __contains__(self, key: object) -> bool: ...
    has_key = __contains__
    def clear(self) -> None: ...
    def copy(self) -> Self: ...
    @overload
    def setdefault(self, key: _KT, default: None = None) -> _VT | None: ...
    @overload
    def setdefault(self, key: _KT, default: _VT) -> _VT: ...
    @overload
    def pop(self, key: _KT) -> _VT: ...
    @overload
    def pop(self, key: _KT, default: _T) -> _VT | _T: ...
    def popitem(self) -> tuple[_KT, _VT]: ...
    @overload  # type: ignore[override]
    def update(self: SupportsGetItem[str, _VT], **kwargs: _VT) -> None: ...
    @overload
    def update(self, m: Collection[tuple[_KT, _VT]], /) -> None: ...
    @overload
    def update(self: SupportsGetItem[str, _VT], m: Collection[tuple[str, _VT]], /, **kwargs: _VT) -> None: ...
    @overload
    def extend(self, other: _SupportsItemsWithIterableResult[_KT, _VT]) -> None: ...
    @overload
    def extend(self: MultiDict[str, _VT], other: _SupportsItemsWithIterableResult[str, _VT], **kwargs: _VT) -> None: ...
    @overload
    def extend(self, other: Iterable[tuple[_KT, _VT]]) -> None: ...
    @overload
    def extend(self: MultiDict[str, _VT], other: Iterable[tuple[str, _VT]], **kwargs: _VT) -> None: ...
    @overload
    def extend(self, other: SupportsKeysAndGetItem[_KT, _VT]) -> None: ...
    @overload
    def extend(self: MultiDict[str, _VT], other: SupportsKeysAndGetItem[str, _VT], **kwargs: _VT) -> None: ...
    @overload
    def extend(self: MultiDict[str, _VT], other: None = None, **kwargs: _VT) -> None: ...
    def __len__(self) -> int: ...
    def keys(self) -> Iterator[_KT]: ...  # type: ignore[override]
    __iter__ = keys
    def items(self) -> Iterator[tuple[_KT, _VT]]: ...  # type: ignore[override]
    def values(self) -> Iterator[_VT]: ...  # type: ignore[override]

class GetDict(MultiDict[str, str]):
    env: WSGIEnvironment
    @overload
    def __init__(self, data: _SupportsItemsWithIterableResult[str, str], env: WSGIEnvironment) -> None: ...
    @overload
    def __init__(self, data: Iterable[tuple[str, str]], env: WSGIEnvironment) -> None: ...
    def on_change(self) -> None: ...
    def __setitem__(self, key: str, value: str) -> None: ...
    def add(self, key: str, value: str) -> None: ...
    def __delitem__(self, key: str) -> None: ...
    def clear(self) -> None: ...
    def setdefault(self, key: str, default: str) -> str: ...  # type: ignore[override]
    @overload
    def pop(self, key: str) -> str: ...
    @overload
    def pop(self, key: str, default: _T) -> str | _T: ...
    def popitem(self) -> tuple[str, str]: ...
    @overload  # type: ignore[override]
    def update(self, **kwargs: str) -> None: ...
    @overload
    def update(self, m: Collection[tuple[str, str]], /, **kwargs: str) -> None: ...
    @overload
    def extend(self, other: _SupportsItemsWithIterableResult[str, str], **kwargs: str) -> None: ...
    @overload
    def extend(self, other: Iterable[tuple[str, str]], **kwargs: str) -> None: ...
    @overload
    def extend(self, other: SupportsKeysAndGetItem[str, str], **kwargs: str) -> None: ...
    @overload
    def extend(self, other: None = None, **kwargs: str) -> None: ...
    def copy(self) -> MultiDict[str, str]: ...  # type: ignore[override]

class NestedMultiDict(MultiDict[_KT, _VT]):
    # FIXME: It would be more accurate to use a Protocol here, which has a
    #        covariant _VT, instead of MultiDict
    dicts: tuple[MultiDict[_KT, _VT], ...]
    def __init__(self, *dicts: MultiDict[_KT, _VT]) -> None: ...
    def __getitem__(self, key: _KT) -> _VT: ...
    # NOTE: These methods all return exceptions, so this will give us
    #       somewhat sane type checker errors, we would prefer to use
    #       something like @type_error here, if it existed.
    #       This is only really necessary, because the inheritance hierachy
    #       is a mess.
    __setitem__: None  # type: ignore[assignment]
    add: None  # type: ignore[assignment]
    __delitem__: None  # type: ignore[assignment]
    clear: None  # type: ignore[assignment]
    setdefault: None  # type: ignore[assignment]
    pop: None  # type: ignore[assignment]
    popitem: None  # type: ignore[assignment]
    update: None  # type: ignore[assignment]
    def getall(self, key: _KT) -> list[_VT]: ...
    def copy(self) -> MultiDict[_KT, _VT]: ...  # type: ignore[override]
    def __contains__(self, key: object) -> bool: ...
    has_key = __contains__
    def __len__(self) -> int: ...
    def items(self) -> Iterator[tuple[_KT, _VT]]: ...  # type: ignore[override]
    def values(self) -> Iterator[_VT]: ...  # type: ignore[override]
    def keys(self) -> Iterator[_KT]: ...  # type: ignore[override]
    __iter__ = keys

class NoVars:
    reason: str
    def __init__(self, reason: str | None = None) -> None: ...
    @overload
    def get(self, key: str, default: None = None) -> None: ...
    @overload
    def get(self, key: str, default: _T) -> _T: ...
    def getall(self, key: str) -> list[str]: ...
    def mixed(self) -> dict[str, str | list[str]]: ...
    def dict_of_lists(self) -> dict[str, list[str]]: ...
    def __contains__(self, key: object) -> Literal[False]: ...
    has_key = __contains__
    def copy(self) -> Self: ...
    def __len__(self) -> Literal[0]: ...
    def __iter__(self) -> Iterator[str]: ...
    def keys(self) -> Iterator[str]: ...
    def values(self) -> Iterator[str]: ...
    def items(self) -> Iterator[tuple[str, str]]: ...
    def __bool__(self) -> Literal[False]: ...



from __future__ import annotations

from _typeshed.wsgi import StartResponse, WSGIApplication, WSGIEnvironment
from collections.abc import Iterable
from typing_extensions import assert_type

from webob.dec import _AnyResponse, wsgify
from webob.request import Request


class App:
    @wsgify
    def __call__(self, request: Request) -> str:
        return "hello"


env: WSGIEnvironment = {}
start_response: StartResponse = lambda x, y, z=None: lambda b: None
application: WSGIApplication = lambda e, s: [b""]
request: Request = Request(env)

x = App()
# since we wsgified our __call__ we should now be a valid WSGIApplication
application = x
assert_type(x(env, start_response), "Iterable[bytes]")
# currently we lose the exact response type, but that should be fine in
# most use-cases, since middlewares operate on an application level, not
# on these raw intermediary functions
assert_type(x(request), _AnyResponse)

# accessing the method from the class should work as you expect it to
assert_type(App.__call__(x, env, start_response), "Iterable[bytes]")
assert_type(App.__call__(x, request), _AnyResponse)


# but we can also wrap it with a middleware that expects to deal with requests
class Middleware:
    @wsgify.middleware
    def restrict_ip(self, req: Request, app: WSGIApplication, ips: list[str]) -> WSGIApplication:
        return app

    __call__ = restrict_ip(x, ips=["127.0.0.1"])


# and we still end up with a valid WSGIApplication
m = Middleware()
application = m
assert_type(m(env, start_response), "Iterable[bytes]")
assert_type(m(request), _AnyResponse)


# the same should work with plain functions
@wsgify
def app(request: Request) -> str:
    return "hello"


application = app
assert_type(app, "wsgify[[], Request]")
assert_type(app(env, start_response), "Iterable[bytes]")
assert_type(app(request), _AnyResponse)
assert_type(app(application), "wsgify[[], Request]")
application = app(application)


@wsgify.middleware
def restrict_ip(req: Request, app: WSGIApplication, ips: list[str]) -> WSGIApplication:
    return app


@restrict_ip(ips=["127.0.0.1"])
@wsgify
def m_app(request: Request) -> str:
    return "hello"


application = m_app
assert_type(m_app, "wsgify[[WSGIApplication], Request]")
assert_type(m_app(env, start_response), "Iterable[bytes]")
assert_type(m_app(request), _AnyResponse)
assert_type(m_app(application), "wsgify[[WSGIApplication], Request]")
application = m_app(application)


# custom request
class MyRequest(Request):
    pass


@wsgify(RequestClass=MyRequest)
def my_request_app(request: MyRequest) -> None:
    pass


application = my_request_app
assert_type(my_request_app, "wsgify[[], MyRequest]")


# we are allowed to accept a less specific request class
@wsgify(RequestClass=MyRequest)
def valid_request_app(request: Request) -> None:
    pass


# but the opposite is not allowed
@wsgify  # type: ignore
def invalid_request_app(request: MyRequest) -> None:
    pass


# we can't really make passing extra arguments directly work
# otherwise we have to give up most of our type safety for
# something that should only be used through wsgify.middleware
wsgify(args=(1,))  # type: ignore
wsgify(kwargs={"ips": ["127.0.0.1"]})  # type: ignore



from _typeshed import StrPath
from collections.abc import Iterable
from typing import overload
from typing_extensions import Self

from unidiff.constants import (
    DEFAULT_ENCODING as DEFAULT_ENCODING,
    LINE_TYPE_ADDED as LINE_TYPE_ADDED,
    LINE_TYPE_CONTEXT as LINE_TYPE_CONTEXT,
    LINE_TYPE_REMOVED as LINE_TYPE_REMOVED,
)
from unidiff.errors import UnidiffParseError as UnidiffParseError

class Line:
    source_line_no: int | None
    target_line_no: int | None
    diff_line_no: int | None
    line_type: str
    value: str
    def __init__(
        self,
        value: str,
        line_type: str,
        source_line_no: int | None = None,
        target_line_no: int | None = None,
        diff_line_no: int | None = None,
    ) -> None: ...
    def __eq__(self, other: Line) -> bool: ...  # type: ignore[override]
    @property
    def is_added(self) -> bool: ...
    @property
    def is_removed(self) -> bool: ...
    @property
    def is_context(self) -> bool: ...

class PatchInfo(list[str]): ...

class Hunk(list[Line]):
    source_start: int
    source_length: int
    target_start: int
    target_length: int
    section_header: str
    def __init__(
        self, src_start: int = 0, src_len: int | None = 0, tgt_start: int = 0, tgt_len: int | None = 0, section_header: str = ""
    ) -> None: ...
    def append(self, line: Line) -> None: ...
    @property
    def added(self) -> int: ...
    @property
    def removed(self) -> int: ...
    def is_valid(self) -> bool: ...
    def source_lines(self) -> Iterable[Line]: ...
    @property
    def source(self) -> Iterable[str]: ...
    def target_lines(self) -> Iterable[Line]: ...
    @property
    def target(self) -> Iterable[str]: ...

class PatchedFile(list[Hunk]):
    patch_info: PatchInfo | None
    source_file: str
    source_timestamp: str | None
    target_file: str
    target_timestamp: str | None
    is_binary_file: bool
    def __init__(
        self,
        patch_info: PatchInfo | None = None,
        source: str = "",
        target: str = "",
        source_timestamp: str | None = None,
        target_timestamp: str | None = None,
        is_binary_file: bool = False,
    ) -> None: ...
    @property
    def path(self) -> str: ...
    @property
    def added(self) -> int: ...
    @property
    def removed(self) -> int: ...
    @property
    def is_rename(self) -> bool: ...
    @property
    def is_added_file(self) -> bool: ...
    @property
    def is_removed_file(self) -> bool: ...
    @property
    def is_modified_file(self) -> bool: ...

class PatchSet(list[PatchedFile]):
    @overload
    def __init__(self, f: Iterable[str] | str, encoding: None = None, metadata_only: bool = False) -> None: ...
    @overload
    def __init__(self, f: Iterable[bytes] | bytes, encoding: str | None = None, metadata_only: bool = False) -> None: ...
    @classmethod
    def from_filename(
        cls, filename: StrPath, encoding: str = "UTF-8", errors: str | None = None, newline: str | None = None
    ) -> Self: ...
    @classmethod
    @overload
    def from_string(cls, data: str, encoding: None = None, errors: str | None = "strict") -> Self: ...
    @classmethod
    @overload
    def from_string(cls, data: bytes, encoding: str | None = None, errors: str | None = "strict") -> Self: ...
    @property
    def added_files(self) -> list[PatchedFile]: ...
    @property
    def removed_files(self) -> list[PatchedFile]: ...
    @property
    def modified_files(self) -> list[PatchedFile]: ...
    @property
    def added(self) -> int: ...
    @property
    def removed(self) -> int: ...



from typing import Final

from braintree.attribute_getter import AttributeGetter

class TransactionLineItem(AttributeGetter):
    class Kind:
        Credit: Final = "credit"
        Debit: Final = "debit"

    def __init__(self, attributes) -> None: ...
    @staticmethod
    def find_all(transaction_id): ...



### This .pyi file is a helper for centralized storage types that are reused across different runtime modules. ###
from _typeshed import FileDescriptor
from collections.abc import Awaitable, Callable, Iterable, MutableMapping
from typing import Any
from typing_extensions import LiteralString, TypeAlias

_StatusType: TypeAlias = str
_HeadersType: TypeAlias = Iterable[tuple[str, str]]

_EnvironType: TypeAlias = MutableMapping[str, Any]  # See https://peps.python.org/pep-0333/
_StartResponseType: TypeAlias = Callable[[_StatusType, _HeadersType], None]
_ResponseBodyType: TypeAlias = Iterable[bytes]
_WSGIAppType: TypeAlias = Callable[[_EnvironType, _StartResponseType], _ResponseBodyType]  # noqa: Y047

_ScopeType: TypeAlias = MutableMapping[str, Any]
_MessageType: TypeAlias = MutableMapping[str, Any]
_ReceiveType: TypeAlias = Callable[[], Awaitable[_MessageType]]
_SendType: TypeAlias = Callable[[_MessageType], Awaitable[None]]
_ASGIAppType: TypeAlias = Callable[[_ScopeType, _ReceiveType, _SendType], Awaitable[None]]  # noqa: Y047

_UnixSocketPathType: TypeAlias = str
_TcpAddressType: TypeAlias = tuple[LiteralString, int]  # noqa: Y047
_AddressType: TypeAlias = _UnixSocketPathType | FileDescriptor | _TcpAddressType  # noqa: Y047



from _typeshed import Incomplete
from logging import Logger

class LanguageValidator:
    logger: Logger | None
    VALID_KEYS: list[str]
    @classmethod
    def get_logger(cls) -> Logger: ...
    @classmethod
    def validate_info(cls, language_id, info: dict[str, Incomplete]) -> bool: ...



from _typeshed import Incomplete
from collections.abc import Iterable

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = [
    "degree_pearson_correlation_coefficient",
    "degree_assortativity_coefficient",
    "attribute_assortativity_coefficient",
    "numeric_assortativity_coefficient",
]

@_dispatchable
def degree_assortativity_coefficient(
    G: Graph[_Node], x: str = "out", y: str = "in", weight: str | None = None, nodes: Iterable[Incomplete] | None = None
) -> float: ...
@_dispatchable
def degree_pearson_correlation_coefficient(
    G: Graph[_Node], x: str = "out", y: str = "in", weight: str | None = None, nodes: Iterable[Incomplete] | None = None
) -> float: ...
@_dispatchable
def attribute_assortativity_coefficient(G: Graph[_Node], attribute: str, nodes: Iterable[Incomplete] | None = None) -> float: ...
@_dispatchable
def numeric_assortativity_coefficient(G: Graph[_Node], attribute: str, nodes: Iterable[Incomplete] | None = None) -> float: ...



import abc
from typing import Literal

from PIL import Image, ImageDraw

from ...._types import Box
from ....main import ActiveWithNeighbors
from ...styledpil import StyledPilImage
from .base import QRModuleDrawer

ANTIALIASING_FACTOR: int

class StyledPilQRModuleDrawer(QRModuleDrawer, metaclass=abc.ABCMeta):
    img: StyledPilImage

class SquareModuleDrawer(StyledPilQRModuleDrawer):
    imgDraw: ImageDraw.ImageDraw
    def drawrect(self, box: Box, is_active: bool) -> None: ...  # type: ignore[override]

class GappedSquareModuleDrawer(StyledPilQRModuleDrawer):
    size_ratio: float
    def __init__(self, size_ratio: float = 0.8) -> None: ...
    imgDraw: ImageDraw.ImageDraw
    delta: float
    def drawrect(self, box: Box, is_active: bool) -> None: ...  # type: ignore[override]

class CircleModuleDrawer(StyledPilQRModuleDrawer):
    circle: Image.Image
    def drawrect(self, box: Box, is_active: bool) -> None: ...  # type: ignore[override]

class RoundedModuleDrawer(StyledPilQRModuleDrawer):
    needs_neighbors: Literal[True]
    radius_ratio: float
    def __init__(self, radius_ratio: float = 1) -> None: ...
    corner_width: int
    SQUARE: Image.Image
    NW_ROUND: Image.Image
    SW_ROUND: Image.Image
    SE_ROUND: Image.Image
    NE_ROUND: Image.Image
    def setup_corners(self) -> None: ...
    def drawrect(self, box: Box, is_active: ActiveWithNeighbors) -> None: ...  # type: ignore[override]

class VerticalBarsDrawer(StyledPilQRModuleDrawer):
    needs_neighbors: Literal[True]
    horizontal_shrink: float
    def __init__(self, horizontal_shrink: float = 0.8) -> None: ...
    half_height: int
    delta: int
    SQUARE: Image.Image
    ROUND_TOP: Image.Image
    ROUND_BOTTOM: Image.Image
    def setup_edges(self) -> None: ...
    def drawrect(self, box: Box, is_active: ActiveWithNeighbors) -> None: ...  # type: ignore[override]

class HorizontalBarsDrawer(StyledPilQRModuleDrawer):
    needs_neighbors: Literal[True]
    vertical_shrink: float
    def __init__(self, vertical_shrink: float = 0.8) -> None: ...
    half_width: int
    delta: int
    SQUARE: Image.Image
    ROUND_LEFT: Image.Image
    ROUND_RIGHT: Image.Image
    def setup_edges(self) -> None: ...
    def drawrect(self, box: Box, is_active: ActiveWithNeighbors) -> None: ...  # type: ignore[override]



from collections.abc import Iterable

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["dominating_set", "is_dominating_set", "connected_dominating_set", "is_connected_dominating_set"]

@_dispatchable
def dominating_set(G: Graph[_Node], start_with: _Node | None = None) -> set[_Node]: ...
@_dispatchable
def is_dominating_set(G: Graph[_Node], nbunch: Iterable[_Node]) -> bool: ...
@_dispatchable
def connected_dominating_set(G: Graph[_Node]) -> set[_Node]: ...
@_dispatchable
def is_connected_dominating_set(G: Graph[_Node], nbunch: Iterable[_Node]) -> bool: ...



from collections.abc import Collection
from typing import Any

import pandas as pd
from shapely import Geometry
from shapely.geometry.base import BaseGeometry

from ..geoseries import GeoSeries

def collect(
    x: Collection[Geometry] | GeoSeries | pd.Series[Any] | Geometry, multi: bool = False  # Cannot use pd.Series[BaseGeometry]
) -> BaseGeometry: ...



import sys

if sys.platform == "darwin":
    from collections.abc import Callable
    from typing import Final, NamedTuple, TypeVar
    from typing_extensions import ParamSpec

    from psutil._common import (
        AccessDenied as AccessDenied,
        NoSuchProcess as NoSuchProcess,
        ZombieProcess as ZombieProcess,
        conn_tmap as conn_tmap,
        conn_to_ntuple as conn_to_ntuple,
        isfile_strict as isfile_strict,
        memoize_when_activated as memoize_when_activated,
        parse_environ_block as parse_environ_block,
        usage_percent as usage_percent,
    )

    from . import _common, _psposix, _psutil_osx, _psutil_posix

    _P = ParamSpec("_P")
    _R = TypeVar("_R")

    __extra__all__: Final[list[str]]
    PAGESIZE: Final[int]
    AF_LINK: Final[int]
    TCP_STATUSES: Final[dict[int, str]]
    PROC_STATUSES: Final[dict[int, str]]
    kinfo_proc_map: Final[dict[str, int]]
    pidtaskinfo_map: Final[dict[str, int]]

    class scputimes(NamedTuple):
        user: float
        nice: float
        system: float
        idle: float

    class svmem(NamedTuple):
        total: int
        available: int
        percent: float
        used: int
        free: int
        active: int
        inactive: int
        wired: int

    class pmem(NamedTuple):
        rss: int
        vms: int
        pfaults: int
        pageins: int

    class pfullmem(NamedTuple):
        rss: int
        vms: int
        pfaults: int
        pageins: int
        uss: int

    def virtual_memory() -> svmem: ...
    def swap_memory() -> _common.sswap: ...
    def cpu_times() -> scputimes: ...
    def per_cpu_times() -> list[scputimes]: ...
    def cpu_count_logical() -> int | None: ...
    def cpu_count_cores() -> int | None: ...
    def cpu_stats() -> _common.scpustats: ...
    def cpu_freq() -> list[_common.scpufreq]: ...

    disk_usage = _psposix.disk_usage
    disk_io_counters = _psutil_osx.disk_io_counters

    def disk_partitions(all: bool = False) -> list[_common.sdiskpart]: ...
    def sensors_battery() -> _common.sbattery | None: ...

    net_io_counters = _psutil_osx.net_io_counters
    net_if_addrs = _psutil_posix.net_if_addrs

    def net_connections(kind: str = "inet") -> list[_common.sconn]: ...
    def net_if_stats() -> dict[str, _common.snicstats]: ...
    def boot_time() -> float: ...
    def users() -> list[_common.suser]: ...
    def pids() -> list[int]: ...

    pid_exists = _psposix.pid_exists

    def is_zombie(pid: int) -> bool: ...
    def wrap_exceptions(fun: Callable[_P, _R]) -> Callable[_P, _R]: ...

    class Process:
        __slots__ = ["_cache", "_name", "_ppid", "pid"]
        pid: int
        def __init__(self, pid: int) -> None: ...
        def oneshot_enter(self) -> None: ...
        def oneshot_exit(self) -> None: ...
        def name(self) -> str: ...
        def exe(self) -> str: ...
        def cmdline(self) -> list[str]: ...
        def environ(self) -> dict[str, str]: ...
        def ppid(self) -> int: ...
        def cwd(self) -> str: ...
        def uids(self) -> _common.puids: ...
        def gids(self) -> _common.puids: ...
        def terminal(self) -> str | None: ...
        def memory_info(self) -> pmem: ...
        def memory_full_info(self) -> pfullmem: ...
        def cpu_times(self) -> _common.pcputimes: ...
        def create_time(self) -> float: ...
        def num_ctx_switches(self) -> _common.pctxsw: ...
        def num_threads(self) -> int: ...
        def open_files(self) -> list[_common.popenfile]: ...
        def net_connections(self, kind: str = "inet"): ...
        def num_fds(self) -> int: ...
        def wait(self, timeout: float | None = None): ...
        def nice_get(self): ...
        def nice_set(self, value): ...
        def status(self) -> str: ...
        def threads(self) -> list[_common.pthread]: ...



from ...extend.operation import ExtendedOperation

class WhoAmI(ExtendedOperation):
    request_name: str
    response_attribute: str
    def config(self) -> None: ...
    def populate_result(self) -> None: ...



from _typeshed import Incomplete
from typing import Any

class Constants:
    @staticmethod
    def get_all_constant_values_from_class(klass: object) -> list[Any]: ...  # Any taken from klass.__dict__
    def get_all_enum_values(enum_class) -> list[Incomplete]: ...



from typing import Final, Protocol, overload, type_check_only

@type_check_only
class _SupportsGet(Protocol):
    @overload
    def get(self, name: str, /) -> str | None: ...
    @overload
    def get(self, name: str, default: str, /) -> str: ...

NO_COLORS: Final[int]
ANSI_8_COLORS: Final[int]
ANSI_16_COLORS: Final[int]
ANSI_256_COLORS: Final[int]
TRUE_COLORS: Final[int]

def detect_color_support(env: _SupportsGet) -> int: ...



from collections.abc import Callable
from logging import Logger
from typing import Any, TypeVar

_F = TypeVar("_F", bound=Callable[..., Any])

def create_log_exception_decorator(logger: Logger) -> Callable[[_F], _F]: ...



from typing import Any

def varlist(name, count): ...
def indent_block(block, padding): ...

BFSTR: Any

def render_encipher(write, indent: int = 0) -> None: ...
def write_encipher_function(write, indent: int = 0) -> None: ...
def write_expand_function(write, indent: int = 0) -> None: ...
def main() -> None: ...



from consolemenu import ConsoleMenu as ConsoleMenu
from consolemenu.console_menu import MenuItem
from consolemenu.items import SubmenuItem as SubmenuItem
from consolemenu.menu_formatter import MenuFormatBuilder

class MultiSelectMenu(ConsoleMenu):
    ack_item_completion: bool
    def __init__(
        self,
        title: str | None = None,
        subtitle: str | None = None,
        formatter: MenuFormatBuilder | None = None,
        prologue_text: str | None = None,
        epilogue_text: str | None = None,
        ack_item_completion: bool = True,
        show_exit_option: bool = True,
        exit_option_text: str = "Exit",
        clear_screen: bool = True,
    ) -> None: ...
    def append_item(self, item: MenuItem) -> None: ...
    current_option: int
    def process_user_input(self) -> None: ...



import sys
from _typeshed import FileDescriptor
from collections.abc import Callable, Sequence
from types import TracebackType
from typing import Any
from typing_extensions import ParamSpec, disjoint_base

from gevent._ffi.loop import _ErrorHandler
from gevent._types import _Callback
from gevent.libev import watcher

# this c extension is only available on posix
if sys.platform != "win32":
    _P = ParamSpec("_P")

    def get_version() -> str: ...
    def get_header_version() -> str: ...
    # the final item in the list could be an integer if one of the backends did not have a string mapping
    def embeddable_backends() -> list[str | int]: ...
    def recommended_backends() -> list[str | int]: ...
    def supported_backends() -> list[str | int]: ...
    def time() -> float: ...
    @disjoint_base
    class loop:
        starting_timer_may_update_loop_time: bool
        error_handler: _ErrorHandler
        @property
        def approx_timer_resolution(self) -> float: ...  # readonly in Cython
        def __init__(self, flags: Sequence[str] | str | int | None = None, default: bool | None = None, ptr: int = 0) -> None: ...
        def destroy(self) -> None: ...
        @property
        def ptr(self) -> int: ...
        @property
        def WatcherType(self) -> type[watcher.watcher]: ...
        @property
        def MAXPRI(self) -> int: ...
        @property
        def MINPRI(self) -> int: ...
        def handle_error(
            self, context: object | None, type: type[BaseException] | None, value: BaseException | None, tb: TracebackType | None
        ) -> None: ...
        def run(self, nowait: bool = False, once: bool = False) -> None: ...
        def reinit(self) -> None: ...
        def ref(self) -> None: ...
        def unref(self) -> None: ...
        def break_(self, how: int = 1) -> None: ...
        def verify(self) -> None: ...
        def now(self) -> float: ...
        def update_now(self) -> None: ...
        update = update_now  # deprecated
        @property
        def default(self) -> bool: ...
        @property
        def iteration(self) -> int: ...
        @property
        def depth(self) -> int: ...
        @property
        def backend_int(self) -> int: ...
        @property
        def backend(self) -> str | int: ...
        @property
        def pendingcnt(self) -> int: ...
        def io(self, fd: FileDescriptor, events: int, ref: bool = True, priority: int | None = None) -> watcher.io: ...
        def closing_fd(self, fd: FileDescriptor) -> bool: ...
        def timer(self, after: float, repeat: float = 0.0, ref: bool = True, priority: int | None = None) -> watcher.timer: ...
        def signal(self, signum: int, ref: bool = True, priority: int | None = None) -> watcher.signal: ...
        def idle(self, ref: bool = True, priority: int | None = None) -> watcher.idle: ...
        def prepare(self, ref: bool = True, priority: int | None = None) -> watcher.prepare: ...
        def check(self, ref: bool = True, priority: int | None = None) -> watcher.check: ...
        def fork(self, ref: bool = True, priority: int | None = None) -> watcher.fork: ...
        def async_(self, ref: bool = True, priority: int | None = None) -> watcher.async_: ...
        def child(self, pid: int, trace: int = 0, ref: bool = True) -> watcher.child: ...
        def install_sigchld(self) -> None: ...
        def reset_sigchld(self) -> None: ...
        def stat(self, path: str, interval: float = 0.0, ref: bool = True, priority: bool | None = None) -> watcher.stat: ...
        # These technically don't allow the functions arguments to be passed in as kwargs
        # but there's no way to express that yet with ParamSpec, however, we would still like
        # to verify that the arguments match
        def run_callback(self, func: Callable[_P, Any], *args: _P.args, **_: _P.kwargs) -> _Callback: ...
        def run_callback_threadsafe(self, func: Callable[_P, Any], *args: _P.args, **_: _P.kwargs) -> _Callback: ...
        def fileno(self) -> FileDescriptor | None: ...
        @property
        def activecnt(self) -> int: ...
        @property
        def sig_pending(self) -> int: ...
        # the final item in the list could be a integer if some of the flags don't a string mapping
        @property
        def origflags(self) -> list[str | int]: ...
        @property
        def origflags_int(self) -> int: ...
        @property
        def sigfd(self) -> FileDescriptor: ...

    __all__ = [
        "get_version",
        "get_header_version",
        "supported_backends",
        "recommended_backends",
        "embeddable_backends",
        "time",
        "loop",
    ]



import sys
from _typeshed import SupportsWrite
from collections.abc import Iterator
from types import TracebackType
from typing import Final, Literal
from typing_extensions import Self

from .biffh import *
from .formatting import XF, Font, Format
from .formula import *
from .sheet import Cell, Sheet
from .timemachine import *

empty_cell: Final[Cell]
MY_EOF: Final[int]
SUPBOOK_UNK: Final[int]
SUPBOOK_INTERNAL: Final[int]
SUPBOOK_EXTERNAL: Final[int]
SUPBOOK_ADDIN: Final[int]
SUPBOOK_DDEOLE: Final[int]
SUPPORTED_VERSIONS: Final[tuple[int, ...]]
builtin_name_from_code: Final[dict[str, str]]
code_from_builtin_name: Final[dict[str, str]]

def open_workbook_xls(
    filename: str | None = None,
    logfile: SupportsWrite[str] = sys.stdout,
    verbosity: int = 0,
    use_mmap: bool = True,
    file_contents: bytes | None = None,
    encoding_override: str | None = None,
    formatting_info: bool = False,
    on_demand: bool = False,
    ragged_rows: bool = False,
    ignore_workbook_corruption: bool = False,
) -> Book: ...

class Name(BaseObject):
    _repr_these: list[str]
    book: Book | None = None
    hidden: Literal[0, 1]
    func: Literal[0, 1]
    vbasic: Literal[0, 1]
    macro: Literal[0, 1]
    complex: Literal[0, 1]
    builtin: Literal[0, 1]
    funcgroup: Literal[0, 1]
    binary: Literal[0, 1]
    name_index: int
    name: str
    raw_formula: bytes
    scope: Literal[-1, -2, -3, 0]
    result: Operand | None
    def cell(self) -> Cell: ...
    def area2d(self, clipped: bool = True) -> tuple[Sheet, int, int, int, int]: ...

class Book(BaseObject):
    nsheets: int
    datemode: Literal[0, 1]
    biff_version: int
    name_obj_list: list[Name]
    codepage: int | None
    encoding: str | None
    countries: tuple[int, int]
    user_name: str
    font_list: list[Font]
    xf_list: list[XF]
    format_list: list[Format]
    format_map: dict[int, Format]
    style_name_map: dict[str, tuple[int, int]]
    colour_map: dict[int, tuple[int, int, int] | None]
    palette_record: list[tuple[int, int, int]]
    load_time_stage_1: float
    load_time_stage_2: float
    def sheets(self) -> list[Sheet]: ...
    def sheet_by_index(self, sheetx: int) -> Sheet: ...
    def __iter__(self) -> Iterator[Sheet]: ...
    def sheet_by_name(self, sheet_name: str) -> Sheet: ...
    def __getitem__(self, item: int | str) -> Sheet: ...
    def sheet_names(self) -> list[str]: ...
    def sheet_loaded(self, sheet_name_or_index: int | str) -> bool: ...
    def unload_sheet(self, sheet_name_or_index: int | str) -> None: ...
    mem: bytes | None = None
    filestr: bytes | None = None
    def release_resources(self) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_value: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...
    name_and_scope_map: dict[tuple[str, int], Name]
    name_map: dict[str, list[Name]]
    raw_user_name: bool
    builtinfmtcount: int
    addin_func_names: list[str]
    def __init__(self) -> None: ...
    logfile: SupportsWrite[str]
    verbosity: int
    use_mmap: bool
    encoding_override: str | None
    formatting_info: bool
    on_demand: bool
    ragged_rows: bool
    stream_len: int
    base: int
    def biff2_8_load(
        self,
        filename: str | None = None,
        file_contents: bytes | None = None,
        logfile: SupportsWrite[str] = sys.stdout,
        verbosity: int = 0,
        use_mmap: bool = True,
        encoding_override: str | None = None,
        formatting_info: bool = False,
        on_demand: bool = False,
        ragged_rows: bool = False,
        ignore_workbook_corruption: bool = False,
    ) -> None: ...
    xfcount: int
    actualfmtcount: int
    def initialise_format_info(self) -> None: ...
    def get2bytes(self) -> int: ...
    def get_record_parts(self) -> tuple[int, int, bytes]: ...
    def get_record_parts_conditional(self, reqd_record: int) -> tuple[int | None, int, bytes]: ...
    def get_sheet(self, sh_number: int, update_pos: bool = True) -> Sheet: ...
    def get_sheets(self) -> None: ...
    def fake_globals_get_sheet(self) -> None: ...
    def handle_boundsheet(self, data: bytes) -> None: ...
    def handle_builtinfmtcount(self, data: bytes) -> None: ...
    def derive_encoding(self) -> str: ...
    def handle_codepage(self, data: bytes) -> None: ...
    def handle_country(self, data: bytes) -> None: ...
    def handle_datemode(self, data: bytes) -> None: ...
    def handle_externname(self, data: bytes) -> None: ...
    def handle_externsheet(self, data: bytes) -> None: ...
    def handle_filepass(self, data: bytes) -> None: ...
    def handle_name(self, data: bytes) -> None: ...
    def names_epilogue(self) -> None: ...
    def handle_obj(self, data: bytes) -> None: ...
    def handle_supbook(self, data: bytes) -> None: ...
    def handle_sheethdr(self, data: bytes) -> None: ...
    def handle_sheetsoffset(self, data: bytes) -> None: ...
    def handle_sst(self, data: bytes) -> None: ...
    def handle_writeaccess(self, data: bytes) -> None: ...
    def parse_globals(self) -> None: ...
    def read(self, pos: int, length: int) -> bytes: ...
    def getbof(self, rqd_stream: int) -> int | None: ...

# Helper functions
def expand_cell_address(inrow: int, incol: int) -> tuple[int, int, int, int]: ...
def display_cell_address(rowx: int, colx: int, relrow: int, relcol: int) -> str: ...
def unpack_SST_table(datatab: list[bytes], nstrings: int) -> tuple[list[str], dict[int, list[tuple[int, int]]]]: ...



from enum import IntEnum
from typing import Final

class UnicodeScript(IntEnum):
    COMMON = 0
    LATIN = 1
    GREEK = 2
    CYRILLIC = 3
    ARMENIAN = 4
    HEBREW = 5
    ARABIC = 6
    SYRIAC = 7
    THAANA = 8
    DEVANAGARI = 9
    BENGALI = 10
    GURMUKHI = 11
    GUJARATI = 12
    ORIYA = 13
    TAMIL = 14
    TELUGU = 15
    KANNADA = 16
    MALAYALAM = 17
    SINHALA = 18
    THAI = 19
    LAO = 20
    TIBETAN = 21
    MYANMAR = 22
    GEORGIAN = 23
    HANGUL = 24
    ETHIOPIC = 25
    CHEROKEE = 26
    CANADIAN_ABORIGINAL = 27
    OGHAM = 28
    RUNIC = 29
    KHMER = 30
    MONGOLIAN = 31
    HIRAGANA = 32
    KATAKANA = 33
    BOPOMOFO = 34
    HAN = 35
    YI = 36
    OLD_ITALIC = 37
    GOTHIC = 38
    DESERET = 39
    INHERITED = 40
    TAGALOG = 41
    HANUNOO = 42
    BUHID = 43
    TAGBANWA = 44
    LIMBU = 45
    TAI_LE = 46
    LINEAR_B = 47
    UGARITIC = 48
    SHAVIAN = 49
    OSMANYA = 50
    CYPRIOT = 51
    BRAILLE = 52
    BUGINESE = 53
    COPTIC = 54
    NEW_TAI_LUE = 55
    GLAGOLITIC = 56
    TIFINAGH = 57
    SYLOTI_NAGRI = 58
    OLD_PERSIAN = 59
    KHAROSHTHI = 60
    BALINESE = 61
    CUNEIFORM = 62
    PHOENICIAN = 63
    PHAGS_PA = 64
    NKO = 65
    SUNDANESE = 66
    LEPCHA = 67
    OL_CHIKI = 68
    VAI = 69
    SAURASHTRA = 70
    KAYAH_LI = 71
    REJANG = 72
    LYCIAN = 73
    CARIAN = 74
    LYDIAN = 75
    CHAM = 76
    TAI_THAM = 77
    TAI_VIET = 78
    AVESTAN = 79
    EGYPTIAN_HIEROGLYPHS = 80
    SAMARITAN = 81
    LISU = 82
    BAMUM = 83
    JAVANESE = 84
    MEETEI_MAYEK = 85
    IMPERIAL_ARAMAIC = 86
    OLD_SOUTH_ARABIAN = 87
    INSCRIPTIONAL_PARTHIAN = 88
    INSCRIPTIONAL_PAHLAVI = 89
    OLD_TURKIC = 90
    KAITHI = 91
    BATAK = 92
    BRAHMI = 93
    MANDAIC = 94
    CHAKMA = 95
    MEROITIC_CURSIVE = 96
    MEROITIC_HIEROGLYPHS = 97
    MIAO = 98
    SHARADA = 99
    SORA_SOMPENG = 100
    TAKRI = 101
    CAUCASIAN_ALBANIAN = 102
    BASSA_VAH = 103
    DUPLOYAN = 104
    ELBASAN = 105
    GRANTHA = 106
    PAHAWH_HMONG = 107
    KHOJKI = 108
    LINEAR_A = 109
    MAHAJANI = 110
    MANICHAEAN = 111
    MENDE_KIKAKUI = 112
    MODI = 113
    MRO = 114
    OLD_NORTH_ARABIAN = 115
    NABATAEAN = 116
    PALMYRENE = 117
    PAU_CIN_HAU = 118
    OLD_PERMIC = 119
    PSALTER_PAHLAVI = 120
    SIDDHAM = 121
    KHUDAWADI = 122
    TIRHUTA = 123
    WARANG_CITI = 124
    AHOM = 125
    ANATOLIAN_HIEROGLYPHS = 126
    HATRAN = 127
    MULTANI = 128
    OLD_HUNGARIAN = 129
    SIGNWRITING = 130
    ADLAM = 131
    BHAIKSUKI = 132
    MARCHEN = 133
    NEWA = 134
    OSAGE = 135
    TANGUT = 136
    MASARAM_GONDI = 137
    NUSHU = 138
    SOYOMBO = 139
    ZANABAZAR_SQUARE = 140
    DOGRA = 141
    GUNJALA_GONDI = 142
    MAKASAR = 143
    MEDEFAIDRIN = 144
    HANIFI_ROHINGYA = 145
    SOGDIAN = 146
    OLD_SOGDIAN = 147
    ELYMAIC = 148
    NANDINAGARI = 149
    NYIAKENG_PUACHUE_HMONG = 150
    WANCHO = 151
    CHORASMIAN = 152
    DIVES_AKURU = 153
    KHITAN_SMALL_SCRIPT = 154
    YEZIDI = 155
    CYPRO_MINOAN = 156
    OLD_UYGHUR = 157
    TANGSA = 158
    TOTO = 159
    VITHKUQI = 160
    KAWI = 161
    NAG_MUNDARI = 162
    UNKNOWN = 999

UNICODE_RANGE_TO_SCRIPT: Final[tuple[tuple[int, int, int]]]

def get_unicode_script(char: str) -> UnicodeScript: ...
